#!/usr/bin/env python3
import numpy as np
import h5py as h5
import argparse
import os

def argParsing():
    parser = argparse.ArgumentParser(description='Read PSI MAS 3D data.')
    parser.add_argument("index",
        type=int,
        help='File index for mas output (e.g. 1 for vr000001.h5)')
    parser.add_argument("-data_dir",
        dest='data_dir',
        required=False,
        type=str,
        help='Directory of mas data (default is current working directory).')
    parser.add_argument("-digits",
        required=False,
        type=int,
        default=6,
        help='Number of digits in filename index (e.g. "3" for vr001.h5).  Default is "6".')
    args = parser.parse_args()
    return args

def psi_read_3d_h5(h5_filename):
    x = np.array([])
    y = np.array([])
    z = np.array([])
    f = np.array([])
    h5file = h5.File(h5_filename, 'r')
    f = h5file['Data']
    dims = f.shape
    ndims = np.ndim(f)
    #Get the scales if they exist:
    for i in range(0,ndims):
        if i == 0:
            if (len(h5file['Data'].dims[0].keys())!=0):
                x = h5file['Data'].dims[0][0]
        elif i == 1:
            if (len(h5file['Data'].dims[1].keys())!=0):
                y = h5file['Data'].dims[1][0]
        elif i == 2:
            if (len(h5file['Data'].dims[2].keys())!=0):
                z = h5file['Data'].dims[2][0]
    x = np.array(x)
    y = np.array(y)
    z = np.array(z)
    f = np.array(f)
    h5file.close()
    return (x,y,z,f)

def psi_write_3d_h5(h5_filename, x, y, z, f):
    h5file = h5.File(h5_filename, 'w')
    h5file.create_dataset("Data", data=f)
    if x is None: 
        x = np.array([], dtype=f.dtype)
        y = np.array([], dtype=f.dtype)
        z = np.array([], dtype=f.dtype)
    if x.any() == None:
        x = np.array([], dtype=f.dtype)
        y = np.array([], dtype=f.dtype)
        z = np.array([], dtype=f.dtype)
    x=x.astype(f.dtype)
    y=y.astype(f.dtype)
    z=z.astype(f.dtype)
    ndims = np.ndim(f)
    for i in range(0,ndims):
        if i == 0 and len(x) != 0:
            dim = h5file.create_dataset("dim1", data=x)
            dim.make_scale('dim1')
            h5file['Data'].dims[0].attach_scale(dim)
            h5file['Data'].dims[0].label = 'dim1'
        if i == 1 and len(y) != 0:
            dim = h5file.create_dataset("dim2", data=y)
            dim.make_scale('dim2')
            h5file['Data'].dims[1].attach_scale(dim)
            h5file['Data'].dims[1].label = 'dim2'
        elif i == 2 and len(z) != 0:
            dim = h5file.create_dataset("dim3", data=z)
            dim.make_scale('dim3')
            h5file['Data'].dims[2].attach_scale(dim)
            h5file['Data'].dims[2].label = 'dim3'
    h5file.close()

def main():

    ## Get input agruments:
    args = argParsing()

    fstr = '{:0'+str(args.digits)+'d}' 
    index_str = fstr.format(args.index)

    if args.data_dir is None:
        args.data_dir = os.getcwd()

    ## Set unit factors:
    vfac    = 481.371067364613509      # km/s
    bfac    = 2.206891397800740        # Gauss
    rhofac  = 1.672600000000000e-16    # g/cm^3
    tempfac = 2.807066716734894e+07    # Kelvin

    ## Read standard MAS quantities (v,b,t,rho):
    filename = args.data_dir+'/vr'+index_str+'.h5'
    rvec_m, tvec_h, pvec_h, vr_raw   = psi_read_3d_h5(filename)
    filename = args.data_dir+'/vt'+index_str+'.h5'
    rvec_h, tvec_m, pvec_h, vt_raw   = psi_read_3d_h5(filename)
    filename = args.data_dir+'/vp'+index_str+'.h5'
    rvec_h, tvec_h, pvec_m, vp_raw   = psi_read_3d_h5(filename)
    filename = args.data_dir+'/br'+index_str+'.h5'
    rvec_h, tvec_m, pvec_m, br_raw   = psi_read_3d_h5(filename)
    filename = args.data_dir+'/bt'+index_str+'.h5'
    rvec_m, tvec_h, pvec_m, bt_raw   = psi_read_3d_h5(filename)
    filename = args.data_dir+'/bp'+index_str+'.h5'
    rvec_m, tvec_m, pvec_h, bp_raw   = psi_read_3d_h5(filename)
    filename = args.data_dir+'/rho'+index_str+'.h5'
    rvec_h, tvec_h, pvec_h, rho_raw  = psi_read_3d_h5(filename)
    filename = args.data_dir+'/t'+index_str+'.h5'
    rvec_h, tvec_h, pvec_h, temp_raw = psi_read_3d_h5(filename)       

    ## Interpolate to MMM mesh and apply unit factors:
    vr   = vfac*0.25*(vr_raw[:-1, :-1, :  ] + \
                      vr_raw[:-1, 1: , :  ] + \
                      vr_raw[1: , :-1, :  ] + \
                      vr_raw[1: , 1: , :  ])

    vt   = vfac*0.25*(vt_raw[:-1, :  , :-1] + \
                      vt_raw[:-1, :  , 1: ] + \
                      vt_raw[1: , :  , :-1] + \
                      vt_raw[1: , :  , 1: ])

    vp   = vfac*0.25*(vp_raw[:  , :-1, :-1] + \
                      vp_raw[:  , :-1, 1: ] + \
                      vp_raw[:  , 1: , :-1] + \
                      vp_raw[:  , 1: , 1: ])

    br   = bfac*0.5*(br_raw[:  , :  , 1: ] + \
                     br_raw[:  , :  , :-1])

    bt   = bfac*0.5*(bt_raw[:  , 1: , :  ] + \
                     bt_raw[:  , :-1, :  ])

    bp   = bfac*0.5*(bp_raw[1: , :  , :  ] + \
                     bp_raw[:-1, :  , :  ])

    rho  = rhofac*0.125*(rho_raw[1: , :-1, :-1] + \
                         rho_raw[1: , 1: , :-1] + \
                         rho_raw[1: , :-1, 1: ] + \
                         rho_raw[1: , 1: , 1: ] + \
                         rho_raw[:-1, :-1, :-1] + \
                         rho_raw[:-1, 1: , :-1] + \
                         rho_raw[:-1, :-1, 1: ] + \
                         rho_raw[:-1, 1: , 1: ])

    temp = tempfac*0.125*(temp_raw[1: , :-1, :-1] + \
                          temp_raw[1: , 1: , :-1] + \
                          temp_raw[1: , :-1, 1: ] + \
                          temp_raw[1: , 1: , 1: ] + \
                          temp_raw[:-1, :-1, :-1] + \
                          temp_raw[:-1, 1: , :-1] + \
                          temp_raw[:-1, :-1, 1: ] + \
                          temp_raw[:-1, 1: , 1: ])

    ## Set scales to main mesh.
    ## These are the scales to use with the interpolated field arrays.

    rvec = rvec_m
    tvec = tvec_m
    pvec = pvec_m

    ########################################################################
    ## At this point, we have vr,vt,vp,br,bt,bp,rho,temp all on the same
    ## grid defined by rvec, tvec, and pvec (radius, co-latitude, longitude).
    ########################################################################

    print(' ')
    print(' Read all MAS files:  '+args.data_dir+'/<FIELD>'+index_str+'.h5')
    print(' ')

    ## Print out units
    print(' ###### UNITS #########################')
    print(' ')
    print(' Velocity       (V):  km/s')
    print(' Magnetic Field (B):  Gauss')
    print(' Density      (RHO):  g/cm^3')            
    print(' Temperature (TEMP):  Kelvin') 
    print(' Radius         (r):  Rs (6.96e10 cm)') 
    print(' Co-latitude    (t):  Radians')
    print(' Longitude      (p):  Radians')    
    print(' ')

    ## Get and print grid information:
    NR = rvec.size
    NT = tvec.size
    NP = pvec.size
    
    print(' NOTE! There is a 1-point overlap at p=0 and p=2*pi')
    print(' ')
    print(' Shape of mas data array: '+str(vr.shape))
    print(' Nr:            '+str(NR))
    print(' Ntheta:        '+str(NT))
    print(' Nphi:          '+str(NP))
    print(' min(r):        '+str(np.min(rvec)))
    print(' max(r):        '+str(np.max(rvec)))
    print(' min(theta):    '+str(np.min(tvec)))
    print(' max(theta):    '+str(np.max(tvec)))
    print(' min(phi):      '+str(np.min(pvec)))
    print(' max(phi):      '+str(np.max(pvec)))
    print(' ')
    print(' Example data point for vr at indecies r_i=3, t_i=4, p_i=5 (vr[5,4,3]): ')
    print(' ')
    print(' {}\t{}\t{}\t{}'.format('r                 ', 'theta             ', 'phi               ', 'value'))
    print(' {}\t{}\t{}\t{}'.format(rvec[3], tvec[4], pvec[5], vr[5,4,3]))
    print(' ')

    print(' ## Example how to loop through whole array in stride-1:')
    print(' ## (i->r, j->t, k->p)')
    print(' ')
    print(' for k in range(NP):')
    print('   for j in range(NT):')
    print('     for i in range(NR):')
    print('       print(vr[k,j,i])')
    print(' ')    

if __name__ == '__main__':
    main()

